@@ -8,43 +8,47 @@
 #' @param x Optional initial design as a list. If NULL (default), \code{creationFunction} (in \code{control} list) is used to create initial design. 
 #' If \code{x} has less individuals than specified by \code{control$evalInit}, \code{creationFunction} will fill up the design.
 #' @param fun target function to be minimized
-#' @param control (list), with the options of optimization and model building approaches employed\cr
-#' \code{evalInit} Number of initial evaluations (i.e., size of the initial design), integer, default is \code{2}\cr
-#' \code{vectorized} Boolean. Defines whether target function is vectorized (takes a list of solutions as argument) or not (takes single solution as argument). Default: FALSE\cr
-#' \code{verbosity} Level of text output during run. Defaults to 0, no output.\cr
-#' \code{plotting} Plot optimization progress during run (TRUE) or not (FALSE). Default is FALSE.\cr
-#' \code{targetY} optimal value to be found, stopping criterion, default is \code{-Inf}\cr
-#' \code{evalBudget} maximum number of target function evaluations, default is \code{100}\cr
-#' \code{creationRetries} When a model does not predict an actually improving solution, a random exploration step is performed. \code{creationRetries} solutions are created randomly. 
+#' @param control (list), with the options of optimization and model building approaches employed:
+#' \describe{
+#' \item{\code{evalInit}}{ Number of initial evaluations (i.e., size of the initial design), integer, default is \code{2}}
+#' \item{\code{vectorized}}{ Boolean. Defines whether target function is vectorized (takes a list of solutions as argument) or not (takes single solution as argument). Default: FALSE}
+#' \item{\code{verbosity}}{ Level of text output during run. Defaults to 0, no output.}
+#' \item{\code{plotting}}{ Plot optimization progress during run (TRUE) or not (FALSE). Default is FALSE.}
+#' \item{\code{targetY}}{ optimal value to be found, stopping criterion, default is \code{-Inf}}
+#' \item{\code{evalBudget}}{ maximum number of target function evaluations, default is \code{100}}
+#' \item{\code{creationRetries}}{ When a model does not predict an actually improving solution, a random exploration step is performed. \code{creationRetries} solutions are created randomly. 
 #' 		For each, distance to all known solutions is calculated. The minimum distance is recorded for each random solution. 
-#' 		The random solution with maximal minimum distance is chosen doe be evaluated in the next iteration.\cr
-#' \code{model} Model to be used as a surrogate of the target function. Default is "K" (Kriging). Also
-#'		available are: "LM" (linear, distance-based model), "RBFN" Radial Basis Function Network.\cr
-#' \code{modelSettings} List of settings for \code{model} building, passed on as the \code{control} argument to the model training functions \code{\link{modelKriging}}, \code{\link{modelLinear}}, \code{\link{modelRBFN}}.\cr
-#' \code{infill} This parameter specifies a function to be used for the infill criterion (e.g., the default is expected improvement \code{infillExpectedImprovement}).
-#' To use no specific infill criterion this has to be set to \code{NA}. Infill criteria are only used with models that may provide some error estimate with predictions.\cr
-#' \code{optimizer} Optimizer that finds the minimum of the surrogate model. Default is \code{\link{optimEA}}, an Evolutionary Algorithm.\cr
-#' \code{optimizerSettings} List of settings (\code{control}) for the \code{optimizer} function.\cr
-#' \code{initialDesign} Design function that generates the initial design. Default is \code{designMaxMinDist}, which creates a design that maximizes the minimum distance between points.\cr
-#' \code{initialDesignSettings} List of settings (\code{control}) for the \code{initialDesign} function.\cr
-#' \code{creationFunction} Function to create individuals/solutions in search space. Default is a function that creates random permutations of length 6\cr
-#' \code{distanceFunction} distanceFunction a suitable distance function of type f(x1,x2), returning a scalar distance value, preferably between 0 and 1.
+#' 		The random solution with maximal minimum distance is chosen doe be evaluated in the next iteration.}
+#' \item{\code{model}}{ Model to be used as a surrogate of the target function. Default is "K" (Kriging). Also
+#'		available are: "LM" (linear, distance-based model), "RBFN" Radial Basis Function Network.}
+#' \item{\code{modelSettings}}{ List of settings for \code{model} building, passed on as the \code{control} argument to the model training functions \code{\link{modelKriging}}, \code{\link{modelLinear}}, \code{\link{modelRBFN}}.}
+#' \item{\code{infill}}{ This parameter specifies a function to be used for the infill criterion (e.g., the default is expected improvement \code{infillExpectedImprovement}).
+#' To use no specific infill criterion this has to be set to \code{NA}, in which case the prediction of the surrogate model is used. Infill criteria are only used with models that may provide some error estimate with predictions.}
+#' \item{\code{optimizer}}{ Optimizer that finds the minimum of the surrogate model. Default is \code{\link{optimEA}}, an Evolutionary Algorithm.}
+#' \item{\code{optimizerSettings}}{ List of settings (\code{control}) for the \code{optimizer} function.}
+#' \item{\code{initialDesign}}{ Design function that generates the initial design. Default is \code{designMaxMinDist}, which creates a design that maximizes the minimum distance between points.}
+#' \item{\code{initialDesignSettings}}{ List of settings (\code{control}) for the \code{initialDesign} function.}
+#' \item{\code{creationFunction}}{ Function to create individuals/solutions in search space. Default is a function that creates random permutations of length 6}
+#' \item{\code{distanceFunction}}{ distanceFunction a suitable distance function of type f(x1,x2), returning a scalar distance value, preferably between 0 and 1.
 #'      Maximum distances larger 1 are not a problem, but may yield scaling bias when different measures are compared.
 #' 		Should be non-negative and symmetric. With the setting \code{control$model="K"} this can also be a list of different fitness functions.
-#'    Default is Hamming distance for permutations: distancePermutationHamming.
+#'    Default is Hamming distance for permutations: distancePermutationHamming.}
+#' }
 #'
-#' @return a list:\cr 	
-#' \code{xbest} best solution found\cr
-#' \code{ybest} fitness of the best solution\cr
-#' \code{x} history of all evaluated solutions\cr
-#' \code{y} corresponding target function values f(x)\cr
-#' \code{fit} model-fit created in the last iteration\cr
-#' \code{fpred} prediction function created in the last iteration\cr
-#' \code{count} number of performed target function evaluations\cr 
-#' \code{message} message string, giving information on termination reason
-#' \code{convergence} error/status code: \code{-1} for termination due 
+#' @return a list:
+#' \describe{
+#' \item{\code{xbest}}{ best solution found}
+#' \item{\code{ybest}}{ fitness of the best solution}
+#' \item{\code{x}}{ history of all evaluated solutions}
+#' \item{\code{y}}{ corresponding target function values f(x)}
+#' \item{\code{fit}}{ model-fit created in the last iteration}
+#' \item{\code{fpred}}{ prediction function created in the last iteration}
+#' \item{\code{count}}{ number of performed target function evaluations}
+#' \item{\code{message}}{ message string, giving information on termination reason}
+#' \item{\code{convergence}}{ error/status code: \code{-1} for termination due 
 #' to failed model building, \code{0} for termination due to depleted budget, 
-#' \code{1} if attained objective value is equal to or below target (\code{control$targetY})
+#' \code{1} if attained objective value is equal to or below target (\code{control$targetY})}
+#' }
 #' 
 #' @examples
 #' seed <- 0
@@ -102,11 +106,13 @@ optimCEGO <- function(x=NULL,fun,control=list()){
       , optimizer = optimEA
 			, optimizerSettings = list()
 			, initialDesign = designMaxMinDist
+			, archiveModelInfo = NULL #TODO document
 			, initialDesignSettings = list())
 	con[names(control)] <- control
 	control<-con
 	rm(con)
 	count <- control$evalInit
+	archiveModelInfo <- control$archiveModelInfo
   vectorized <- control$vectorized
   verbosity <- control$verbosity
   plotting <- control$plotting
@@ -124,18 +130,23 @@ optimCEGO <- function(x=NULL,fun,control=list()){
 		fn <- fun
 	
 	## Create main object of this function, which will also be the return value	
-	res <- list(xbest=NA, ybest=NA, x=NA,y=NA,fit=NA,fpred=NA,distances=NA,count=count)
+	res <- list(xbest=NA, ybest=NA, x=NA,y=NA,distances=NA,modelArchive=NA,count=count,convergence=0,message="")
 	
   ## Termination information:
   msg <- "Termination message:"
-  res$convergence <- 0 #error/status code related to msg
   
 	## Create initial design of experiment
 	res$x <- control$initialDesign(x,creationFunction,count,control$initialDesignSettings)
+
+	## Calculate distances between samples. If distance function has parameters: do not calculate.
+	if(length(distanceFunction)==1)
+		distanceHasParam <- length(formalArgs(distanceFunction))>2
+	else
+		distanceHasParam <- any(sapply(sapply(distanceFunction,formalArgs),length) > 2)
+	if(!distanceHasParam)
+		res$distances <- distanceMatrixWrapper(res$x,distanceFunction)
+
 	
-	## Calculate distances between samples						TODO: what if distance function has parameters. see also kernel parameters. has to be calculated on the fly. switch off here?
-	res$distances <- distanceMatrixWrapper(res$x,distanceFunction)
-		
 	## Evaluate initial population	
 	res$y <- fn(res$x)
 
@@ -146,6 +157,19 @@ optimCEGO <- function(x=NULL,fun,control=list()){
 	
 	## build initial model
 	model <- buildModel(res,distanceFunction,control)
+		
+	## archive desired model information
+	if(!is.null(archiveModelInfo)){
+		res$modelArchive <- list()
+		archiveIndex <- 1
+		if(identical(model,NA)){
+			res$modelArchive[[archiveIndex]] <- rep(NA,length(archiveModelInfo))
+			names(res$modelArchive[[archiveIndex]]) <- archiveModelInfo
+		}else{ #todo!			
+			res$modelArchive[[archiveIndex]] <- model$fit[archiveModelInfo]
+			names(res$modelArchive[[archiveIndex]]) <- archiveModelInfo
+		}		
+	}
 
 	## check whether EI infill is used
 	useEI <- is.function(control$infill)
@@ -193,12 +217,25 @@ optimCEGO <- function(x=NULL,fun,control=list()){
 			plot(res$y,type="l",xlab="number of evaluations", ylab="y")
 			abline(res$ybest,0,lty=2)
 		}
-
+	
 		## Update the distance matrix				#TODO what if distance parameters?
-		res$distances <- distanceMatrixUpdate(res$distances,res$x,distanceFunction)
+		if(!distanceHasParam)
+			res$distances <- distanceMatrixUpdate(res$distances,res$x,distanceFunction)
 	
 		## Update surrogate model and prediction function:
 		model <- buildModel(res,distanceFunction,control)
+		
+		## archive desired model information
+		if(!is.null(archiveModelInfo)){
+			archiveIndex <- archiveIndex+1
+			if(identical(model,NA)){
+				res$modelArchive[[archiveIndex]] <- rep(NA,length(archiveModelInfo))
+				names(res$modelArchive[[archiveIndex]]) <- archiveModelInfo
+			}else{
+				res$modelArchive[[archiveIndex]] <- model$fit[archiveModelInfo]
+				names(res$modelArchive[[archiveIndex]]) <- archiveModelInfo
+			}
+		}
 	}
   
   #stopping criteria information for user:
@@ -209,7 +246,7 @@ optimCEGO <- function(x=NULL,fun,control=list()){
     msg <- paste(msg,"Target function evaluation budget depleted.")		
   }
   res$message <- msg
-  
+  res$distances <- NULL
 	res #return
 }
 
@@ -225,16 +262,20 @@ optimCEGO <- function(x=NULL,fun,control=list()){
 #'      Maximum distances larger 1 are no problem, but may yield scaling bias when different measures are compared.
 #' 		Should be non-negative and symmetric.  In case Kriging is chosen, it can also be a list of several distance functions. In this case, MLE is used 
 #'		to determine the most suited distance measure (see the last reference).
-#' @param control list of options \cr
-#' \code{model} Model to be used as a surrogate of the target function. Default is "K" (Kriging). Also
-#'		available are: "LM" (linear, distance-based model), "RBFN" Radial Basis Function Network.\cr
-#' \code{modelSettings} List of settings for model building, passed on as the control argument to the model training functions \code{\link{modelKriging}}, \code{\link{modelLinear}}, \code{\link{modelRBFN}}.
-#' \code{infill} This parameter specifies a function to be used for the infill criterion (e.g., the default is expected improvement \code{infillExpectedImprovement}).
-#' To use no specific infill criterion this has to be set to \code{NA}. Infill criteria are only used with models that may provide some error estimate with predictions.\cr
+#' @param control list with options:
+#' \describe{
+#' \item{\code{model}}{ Model to be used as a surrogate of the target function. Default is "K" (Kriging). Also
+#'		available are: "LM" (linear, distance-based model), "RBFN" Radial Basis Function Network.}
+#' \item{\code{modelSettings}}{ List of settings for model building, passed on as the control argument to the model training functions \code{\link{modelKriging}}, \code{\link{modelLinear}}, \code{\link{modelRBFN}}.}
+#' \item{\code{infill}}{ This parameter specifies a function to be used for the infill criterion (e.g., the default is expected improvement \code{infillExpectedImprovement}).
+#' To use no specific infill criterion this has to be set to \code{NA}. Infill criteria are only used with models that may provide some error estimate with predictions.}
+#' }
 #'
-#' @return a list:\cr 	
-#' \code{fit} model-fit \cr
-#' \code{fpred} prediction function
+#' @return a list:
+#' \describe{
+#' \item{\code{fit}}{ model-fit }
+#' \item{\code{fpred}}{ prediction function}
+#' }
 #' 
 #' @seealso \code{\link{optimCEGO}} 
 #'
@@ -255,6 +296,7 @@ buildModel <- function(res,distanceFunction,control){
 		control$model <- modelKriging
 	
 	fit<-try(control$model(x,y,distanceFunction,control$modelSettings),TRUE)
+	#fit <- control$model(x,y,distanceFunction,control$modelSettings)
 	if(class(fit) == "try-error"){
     #warning("Model building in optimCEGO failed.") #same warning given in optimCEGO function
     return(NA)
@@ -298,6 +340,8 @@ optimizeModel <- function(res,creationFunction,model,control){
 	}
 	if(is.null(control$optimizerSettings$creationFunction))
 		control$optimizerSettings$creationFunction <- creationFunction
+	if(is.null(control$optimizerSettings$vectorized))
+		control$optimizerSettings$vectorized <- TRUE		
 	optimres <- control$optimizer(NULL,model$fpred,control$optimizerSettings)
 	optimres$fpredbestKnownY <- model$fpred(res$xbest) 
 	optimres
